__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup
import re


# parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of description page
# return: 'row' that contains a variety of lists that each hold info on the description page
def vortex_description_parser(soup):
    # Fields to be parsed

    vendor = "-1"  # 0 *Vendor_Name y
    success = "-1"  # 1 Vendor_Successful_Transactions n
    rating_vendor = "-1"  # 2 Vendor_Rating y
    name = "-1"  # 3 *Product_Name y
    describe = "-1"  # 4 Product_Description y
    CVE = "-1"  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"  # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"  # 7 Product_Category y
    views = "-1"  # 8 Product_Number_Of_Views y
    reviews = "-1"  # 9 Product_Number_Of_Reviews n
    rating_item = "-1"  # 10 Product_Rating n
    addDate = "-1"  # 11 Product_AddedDate y
    BTC = "-1"  # 12 Product_BTC_SellingPrice y
    USD = "-1"  # 13 Product_USD_SellingPrice y
    EURO = "-1"  # 14 Product_EURO_SellingPrice n
    sold = "-1"  # 15 Product_QuantitySold n
    left = "-1"  # 16 Product_QuantityLeft n
    shipFrom = "-1"  # 17 Product_ShippedFrom n
    shipTo = "-1"  # 18 Product_ShippedTo n
    image = "-1"  # 19 Product_Image n
    vendor_image = "-1"  # 20 Vendor_Image n

    temp2 = soup.find('div', {'class', 'col-auto font-weight-bold'})

    # Finding Vendor - check
    vendor = temp2.find('a').text
    vendor = cleanString(vendor).strip()

    # Finding Vendor Rating (bug in their system shows standard rating)
    try:
        rating_vendor = temp2.find('span', {'class': "badge badge-pill mr-2"}).text
        rating_vendor = rating_vendor.replace('VendorLvl:', '')
    except:
        rating_vendor = '-1'
    rating_vendor = cleanNumbers(rating_vendor).strip()

    # Finding Product Name - check
    name = soup.find('h3', {'class': "d-inline-block font-weight-bold"}).text
    name = cleanString(name).strip()

    # Finding Product description - check
    try:
        describe = soup.find('p', {'class': 'description pl-3 py-2 border rounded-3'}).text
        describe = cleanString(describe).strip()
    except:
        describe = '-1'

    # print(describe)
    # Finding category - checl
    div_category = soup.find('ol', {'class': "breadcrumb breadcrumb-navbar py-0 px-2 px-md-4 m-0 ml-2 mx-auto justify-content-center h-auto d-flex bg-transparent"}).find_all('li')
    category = div_category[1].find('a').text
    category = cleanString(category).strip()

    # Product rating - check
    try:
        rating = temp2.find('span', {'class': 'text-success mr-2 py-1'}).text
        rating = re.sub(r'[()+%]', '', rating)
    except:
        rating = '-1'
    rating_item = cleanString(rating).strip()

    # Finding BTC and USD/GOLD
    div_price = soup.find('h5', {'class': "product-price"}).find('span', {'class', 'small product-price ml-2'}).text
    div_price = div_price.split('|')

    bit = div_price[0]
    if 'btc' in bit:
        bit = bit.replace('btc', '')
        bit = cleanString(bit).strip()

    BTC = bit

    usd = div_price[2]
    if 'usdt_tron' in usd:
        usd = usd.replace('usdt_tron', '')
        usd = cleanString(usd).strip()
    USD = usd

    left = soup.find('div', {'class', 'row mt-3'}).find('div', {'class', 'col-6 font-weight-bold'}).text
    left = cleanString(left).strip()

    try:
        image = soup.find('li', {'class', 'carousel__slide'}).find('img').get('src').split('base64,')[-1]
    except:
        image = '-1'


    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo, image, vendor_image)

    # Sending the results
    return row


# parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of listing page
# return: 'row' that contains a variety of lists that each hold info on the listing page
def vortex_listing_parser(soup):
    # Fields to be parsed
    nm = 0  # *Total_Products (Should be Integer)
    mktName = "Vortex"  # 0 *Marketplace_Name y
    vendor = []  # 1 *Vendor y
    rating_vendor = []  # 2 Vendor_Rating y
    success = []  # 3 Vendor_Successful_Transactions n
    name = []  # 4 *Product_Name y
    CVE = []  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures) dont worry about this
    MS = []  # 6 Product_MS_Classification (Microsoft Security) dont worry about this
    category = []  # 7 Product_Category y
    describe = []  # 8 Product_Description n
    views = []  # 9 Product_Number_Of_Views y
    reviews = []  # 10 Product_Number_Of_Reviews n
    rating_item = []  # 11 Product_Rating n
    addDate = []  # 12 Product_AddDate y
    BTC = []  # 13 Product_BTC_SellingPrice y
    USD = []  # 14 Product_USD_SellingPrice y
    EURO = []  # 15 Product_EURO_SellingPrice n
    sold = []  # 16 Product_QuantitySold n
    qLeft = []  # 17 Product_QuantityLeft n
    shipFrom = []  # 18 Product_ShippedFrom n
    shipTo = []  # 19 Product_ShippedTo n
    image = []  # 20 Product_Image n
    image_vendor = []  # 21 Vendor_Image n
    href = []  # 22 Product_Links y

    temp = soup.find('main', {'id': 'main'}).find('section', {'id':'page_container'})
    listings = temp.findAll('div', {"class": "product-card col-sm-6 col-md-3 col-xl-4 mb-5"})

    cat = soup.find('section', {'class': 'row px-md-4 mx-0 mb-3'}).find('ol').find_all('li')
    cat = cat[1].find('a').text

    # Populating the Number of Products
    nm = len(listings)

    for listing in listings:
        listing = listing.find('div', {'class': 'product-details'})

        # Finding vendor name - checked
        vendor_name = listing.find('span', {'class': 'd-inline-block w-100 mb-1'}).find('a').text
        if 'ships from' in vendor_name:
            reg = re.compile(r'(.+?)\'s shop')
            vendor_name = reg.match(vendor_name)
            if vendor_name:
                vendor_name = vendor_name.group(1)
            vendor_name = vendor_name.replace('\'s shop', '')
        vendor_name = vendor_name.strip()
        vendor.append(vendor_name)

        # Finding the vendor rating - checked
        vendor_level = listing.find('span', {'class': "badge badge-pill mr-2 font-weight-normal"}).text
        vendor_level = vendor_level.split(' ')
        vendor_level = cleanNumbers(vendor_level[-1])
        rating_vendor.append(vendor_level)

        # Finding the product name - checked
        product_name = listing.find('h4').find('a').text
        product_name = cleanString(product_name).strip()
        name.append(product_name)

        # Searching for CVE and MS categories
        cve = listing.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
        if not cve:
            cveValue = "-1"
        else:
            cee = " "
            for idx in cve:
                cee += (idx)
                cee += "  "
                cee = cee.replace(',', ' ')
                cee = cee.replace('\n', '')
                cee = cee.replace('\t', '')
            cveValue = cee
        CVE.append(cveValue)

        ms = listing.findAll(text=re.compile('MS\d{2}-\d{3}'))
        if not ms:
            MSValue = "-1"
        else:
            me = " "
            for im in ms:
                me += (im)
                me += " "
                me = me.replace(',', ' ')
                me = me.replace('\n', '')
            MSValue = me
        MS.append(MSValue)

        # Finding the category - check
        category_text = cleanString(cat).strip()
        category.append(category_text)

        # Finding the hrefs - check
        description_link = listing.find('h4').find('a')['href']
        href.append(description_link)

        # Finding the views - check
        views_text = '-1'
        views.append(views_text)

        # Finding the date added
        date = '-1'
        # date = datetime.strptime(date, "%d-%m-%Y")
        addDate.append(date)

        # EURO
        EURO.append('-1')

        # Finding the BTC and USD/GOLD
        try:
            money = listing.find('div', {"class": 'w-100 small product-price mb-1 text-right'}).text
            money = money.split('|')
            if 'btc' in money[0]:
                btc_price = money[0].replace('btc', '')
                btc_price = cleanString(btc_price).strip()
        except:
            btc_price = '-1'
        BTC.append(btc_price)

        try:
            if 'usd' in money[2]:
                usd_price = money[2].strip().replace('usdt_tron', '')
                usd_price = cleanString(usd_price).strip()
        except:
            usd_price = '-1'
        USD.append(usd_price)

        sold.append('-1')
        qLeft.append('-1')
        shipTo.append('-1')

        shipf = listing.find('span', {'class': 'd-inline-block w-100 mb-1'}).find('a').text
        if 'ships from' in shipf:
            pattern = re.compile(r"ships from (.*)")
            shipf = pattern.search(shipf)
            if shipf:
                shipf = shipf.group(1)
            shipf = shipf.replace('ships from', '')
        else:
            shipf = '-1'
        shipf = cleanString(shipf).strip()
        shipFrom.append(shipf)

        try:
            temp = listing.find('p', {'class', 'w-100 mb-2'}).find('img').get('src').split('base64,')[-1]
        except:
            temp = '-1'
        image.append(temp)
        image_vendor.append('-1')

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href, image,
                            image_vendor)


# called by the crawler to get description links on a listing page
# @param: beautifulsoup object that is using the correct html page (listing page)
# return: list of description links from a listing page
def vortex_links_parser(soup):
    # Returning all links that should be visited by the Crawler

    href = []
    listings = soup.find('main').findAll('div', {"class": "product-card col-sm-6 col-md-3 col-xl-4 mb-5"})

    for listing in listings:
        # Adding the url to the list of urls
        description_link = listing.find('h4').find('a')['href']
        href.append(description_link)

    return href
